@@ -2,7 +2,9 @@ use coreclr_hosting_shared::{char_t, size_t};
 use dlopen::symbor::{SymBorApi, Symbol};
 
 use crate::{
-    hostfxr_delegate_type, hostfxr_error_writer_fn, hostfxr_handle, hostfxr_initialize_parameters,
+    hostfxr_delegate_type, hostfxr_error_writer_fn, hostfxr_get_available_sdks_result_fn,
+    hostfxr_handle, hostfxr_initialize_parameters,
+    hostfxr_resolve_sdk2_result_fn,
 };
 
 /// [`dlopen::symbor`] abstraction for the hostfxr library.
@@ -16,7 +18,59 @@ pub struct Hostfxr<'lib> {
     /// This function does not return until the application completes execution.
     /// It will shutdown CoreCLR after the application executes.
     /// If the application is successfully executed, this value will return the exit code of the application. Otherwise, it will return an error code indicating the failure.
-    pub hostfxr_main: Symbol<'lib, unsafe extern "C" fn(argc: i32, argv: *const *const char_t) -> i32>,
+    #[cfg(feature = "netcore1_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore1_0")))]
+    pub hostfxr_main:
+        Symbol<'lib, unsafe extern "C" fn(argc: i32, argv: *const *const char_t) -> i32>,
+
+    /// Determines the directory location of the SDK accounting for
+    /// `global.json` and multi-level lookup policy.
+    ///
+    /// Invoked via MSBuild SDK resolver to locate SDK props and targets
+    /// from an msbuild other than the one bundled by the CLI.
+    ///
+    /// # Arguments
+    ///  * `exe_dir`
+    ///      The main directory where SDKs are located in `sdk\[version]`
+    ///      sub-folders. Pass the directory of a dotnet executable to
+    ///      mimic how that executable would search in its own directory.
+    ///      It is also valid to pass nullptr or empty, in which case
+    ///      multi-level lookup can still search other locations if
+    ///      it has not been disabled by the user's environment.
+    ///
+    ///  * `working_dir`
+    ///      The directory where the search for `global.json` (which can
+    ///      control the resolved SDK version) starts and proceeds
+    ///      upwards.
+    ///
+    ///  * `buffer`
+    ///      The buffer where the resolved SDK path will be written.
+    ///
+    ///  * `buffer_size`
+    ///      The size of the buffer argument in [`char_t`] units.
+    ///
+    /// # Return value:
+    ///  * `<0` - Invalid argument
+    ///  * `0`  - SDK could not be found.
+    ///  * `>0` - The number of characters (including null terminator)
+    ///        required to store the located SDK.
+    ///
+    /// If resolution succeeds and the positive return value is less than
+    /// or equal to `buffer_size` (i.e. the the buffer is large enough),
+    /// then the resolved SDK path is copied to the buffer and null
+    /// terminated. Otherwise, no data is written to the buffer.
+    #[deprecated(note = "Use `hostfxr_resolve_sdk2` instead.")]
+    #[cfg(feature = "netcore2_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore2_0")))]
+    pub hostfxr_resolve_sdk: Symbol<
+        'lib,
+        unsafe extern "C" fn(
+            exe_dir: *const char_t,
+            working_dir: *const char_t,
+            buffer: *mut char_t,
+            buffer_size: i32,
+        ) -> i32,
+    >,
 
     /// Run an application.
     ///
@@ -33,6 +87,8 @@ pub struct Hostfxr<'lib> {
     /// This function does not return until the application completes execution.
     /// It will shutdown CoreCLR after the application executes.
     /// If the application is successfully executed, this value will return the exit code of the application. Otherwise, it will return an error code indicating the failure.
+    #[cfg(feature = "netcore2_1")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore2_1")))]
     pub hostfxr_main_startupinfo: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -44,6 +100,11 @@ pub struct Hostfxr<'lib> {
         ) -> i32,
     >,
 
+    #[cfg(all(feature = "netcore2_1", feature = "undocumented"))]
+    #[cfg_attr(
+        all(feature = "doc-cfg", nightly),
+        doc(cfg(feature = "netcore2_1", feature = "undocumented"))
+    )]
     pub hostfxr_main_bundle_startupinfo: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -56,6 +117,76 @@ pub struct Hostfxr<'lib> {
         ) -> i32,
     >,
 
+    /// Determine the directory location of the SDK, accounting for `global.json` and multi-level lookup policy.
+    ///
+    /// # Arguments
+    ///  * `exe_dir` - main directory where SDKs are located in `sdk\[version]` sub-folders.
+    ///  * `working_dir` - directory where the search for `global.json` will start and proceed upwards
+    ///  * `flags` - flags that influence resolution:
+    ///         `disallow_prerelease` - do not allow resolution to return a pre-release SDK version unless a pre-release version was specified via `global.json`
+    ///  * `result` - callback invoked to return resolved values.
+    ///         The callback may be invoked more than once. Strings passed to the callback are valid only for the duration of the call.
+    ///
+    /// If resolution succeeds, result will be invoked with [`resolved_sdk_dir`](crate::hostfxr_resolve_sdk2_result_key_t::resolved_sdk_dir) key and the value will hold the path to the resolved SDK directory.
+    /// If resolution does not succeed, result will be invoked with [`resolved_sdk_dir`](crate::hostfxr_resolve_sdk2_result_key_t::resolved_sdk_dir) key and the value will be [`ptr::null()`](core::ptr::null).
+    ///
+    /// If `global.json` is used, result will be invoked with [`global_json_path`](crate::hostfxr_resolve_sdk2_result_key_t::global_json_path) key and the value will hold the path to `global.json`.
+    /// If there was no `global.json` found, or the contents of `global.json` did not impact resolution (e.g. no version specified), then result will not be invoked with [`global_json_path`](crate::hostfxr_resolve_sdk2_result_key_t::global_json_path) key.
+    #[cfg(feature = "netcore2_1")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore2_1")))]
+    pub hostfxr_resolve_sdk2: Symbol<
+        'lib,
+        unsafe extern "C" fn(
+            exe_dir: *const char_t,
+            working_dir: *const char_t,
+            flags: i32,
+            result: hostfxr_resolve_sdk2_result_fn,
+        ) -> i32,
+    >,
+
+    /// Get the list of all available SDKs ordered by ascending version.
+    ///
+    /// # Arguments
+    ///  * `exe_dir` - path to the dotnet executable
+    ///  * `result` - callback invoked to return the list of SDKs by their directory paths.
+    ///             String array and its elements are valid only for the duration of the call.
+    #[cfg(feature = "netcore2_1")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore2_1")))]
+    pub hostfxr_get_available_sdks: Symbol<
+        'lib,
+        unsafe extern "C" fn(
+            exe_dir: *const char_t,
+            result: hostfxr_get_available_sdks_result_fn,
+        ) -> i32,
+    >,
+
+    /// Get the native search directories of the runtime based upon the specified app.
+    ///
+    /// # Arguments
+    ///  * `argc`,`argv` - command-line arguments
+    ///  * `buffer` - buffer to populate with the native search directories (including a null terminator).
+    ///  * `buffer_size` - size of `buffer` in [`char_t`] units
+    ///  * `required_buffer_size` - if buffer is too small, this will be populated with the minimum required buffer size (including a null terminator). Otherwise, this will be set to 0.
+    ///
+    /// The native search directories will be a list of paths separated by [`PATH_SEPARATOR`], which is a semicolon (;) on Windows and a colon (:) otherwise.
+    ///
+    /// If `buffer_size` is less than the minimum required buffer size, this function will return [`HostApiBufferTooSmall`] and buffer will be unchanged.
+    ///
+    /// [`HostApiBufferTooSmall`]: coreclr_hosting_shared::StatusCode::HostApiBufferTooSmall
+    /// [`PATH_SEPARATOR`]: crate::PATH_SEPARATOR
+    #[cfg(feature = "netcore2_1")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore2_1")))]
+    pub hostfxr_get_native_search_directories: Symbol<
+        'lib,
+        unsafe extern "C" fn(
+            argc: i32,
+            argv: *const *const char_t,
+            buffer: *mut char_t,
+            buffer_size: i32,
+            required_buffer_size: *mut i32,
+        ) -> i32,
+    >,
+
     /// Sets a callback which is to be used to write errors to.
     ///
     /// # Arguments
@@ -79,6 +210,8 @@ pub struct Hostfxr<'lib> {
     /// If the hostfxr invokes functions in hostpolicy as part of its operation, the error writer
     /// will be propagated to hostpolicy for the duration of the call. This means that errors from
     /// both hostfxr and hostpolicy will be reporter through the same error writer.
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_set_error_writer: Symbol<
         'lib,
         unsafe extern "C" fn(error_writer: hostfxr_error_writer_fn) -> hostfxr_error_writer_fn,
@@ -113,6 +246,8 @@ pub struct Hostfxr<'lib> {
     ///
     /// [`Success`]: coreclr_hosting_shared::StatusCode::Success
     /// [`HostInvalidState`]: coreclr_hosting_shared::StatusCode::HostInvalidState
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_initialize_for_dotnet_command_line: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -161,6 +296,8 @@ pub struct Hostfxr<'lib> {
     /// [`Success_HostAlreadyInitialized`]: coreclr_hosting_shared::StatusCode::Success_HostAlreadyInitialized
     /// [`Success_DifferentRuntimeProperties`]: coreclr_hosting_shared::StatusCode::Success_DifferentRuntimeProperties
     /// [`CoreHostIncompatibleConfig`]: coreclr_hosting_shared::StatusCode::CoreHostIncompatibleConfig
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_initialize_for_runtime_config: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -195,6 +332,8 @@ pub struct Hostfxr<'lib> {
     ///
     /// [`hostfxr_set_runtime_property_value`]: struct.HostfxrLib.html#method.hostfxr_set_runtime_property_value
     /// [`hostfxr_close`]: struct.HostfxrLib.html#method.hostfxr_close
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_get_runtime_property_value: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -222,6 +361,8 @@ pub struct Hostfxr<'lib> {
     ///
     /// If the property already exists in the host context, it will be overwritten. If value is [`null`](core::ptr::null()), the
     /// property will be removed.
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_set_runtime_property_value: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -260,6 +401,8 @@ pub struct Hostfxr<'lib> {
     ///
     /// [`hostfxr_set_runtime_property_value`]: struct.HostfxrLib.html#hostfxr_set_runtime_property_value
     /// [`hostfxr_close`]: struct.HostfxrLib.html#method.hostfxr_closee
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_get_runtime_properties: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -286,7 +429,10 @@ pub struct Hostfxr<'lib> {
     ///
     /// [`hostfxr_initialize_for_runtime_config`]: struct.HostfxrLib.html#method.hostfxr_initialize_for_runtime_config
     /// [`hostfxr_initialize_for_dotnet_command_line`]: struct.HostfxrLib.html#method.hostfxr_initialize_for_dotnet_command_line
-    pub hostfxr_run_app: Symbol<'lib, unsafe extern "C" fn(host_context_handle: hostfxr_handle) -> i32>,
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
+    pub hostfxr_run_app:
+        Symbol<'lib, unsafe extern "C" fn(host_context_handle: hostfxr_handle) -> i32>,
 
     /// Gets a typed delegate from the currently loaded CoreCLR or from a newly created one.
     ///
@@ -313,6 +459,8 @@ pub struct Hostfxr<'lib> {
     /// [`hdt_get_function_pointer`]: hostfxr_delegate_type::hdt_get_function_pointer
     /// [`hostfxr_initialize_for_runtime_config`]: struct.HostfxrLib.html#method.hostfxr_initialize_for_runtime_config
     /// [`hostfxr_initialize_for_dotnet_command_line`]: struct.HostfxrLib.html#method.hostfxr_initialize_for_dotnet_command_line
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
     pub hostfxr_get_runtime_delegate: Symbol<
         'lib,
         unsafe extern "C" fn(
@@ -330,5 +478,8 @@ pub struct Hostfxr<'lib> {
     ///
     /// # Return value:
     /// The error code result.
-    pub hostfxr_close: Symbol<'lib, unsafe extern "C" fn(host_context_handle: hostfxr_handle) -> i32>,
+    #[cfg(feature = "netcore3_0")]
+    #[cfg_attr(all(feature = "doc-cfg", nightly), doc(cfg(feature = "netcore3_0")))]
+    pub hostfxr_close:
+        Symbol<'lib, unsafe extern "C" fn(host_context_handle: hostfxr_handle) -> i32>,
 }
